@@ -2,6 +2,12 @@
     begin                : Sat Feb 14 2004
     copyright            : (C) 2004 by Scott Wheeler
     email                : wheeler@kde.org
+
+    copyright            : (C) 2007 by Matthias Kretz
+    email                : kretz@kde.org
+
+    copyright            : (C) 2008 by Michael Pyne
+    email                : michael.pyne@kdemail.net
 ***************************************************************************/
 
 /***************************************************************************
@@ -13,28 +19,28 @@
  *                                                                         *
  ***************************************************************************/
 
-/**
- * Note to those who work here.  The preprocessor variables HAVE_ARTS and HAVE_GSTREAMER
- * are ::ALWAYS DEFINED::.  You can't use #ifdef to see if they're present, you should just
- * use #if.
- *
- * However, HAVE_AKODE is #define'd if present, and undefined if not present.
- * - mpyne
- */
+#include "playermanager.h"
+#include <config.h>
 
 #include <kdebug.h>
+#include <kmessagebox.h>
 #include <klocale.h>
+#include <kactioncollection.h>
+#include <kselectaction.h>
+#include <ktoggleaction.h>
+#include <kurl.h>
 
-#include <qslider.h>
-#include <qtimer.h>
+#include <Phonon/AudioOutput>
+#include <Phonon/MediaObject>
+#include <Phonon/VolumeFaderEffect>
+
+#include <QPixmap>
+#include <QTimer>
 
 #include <math.h>
 
-#include "artsplayer.h"
-#include "akodeplayer.h"
-#include "gstreamerplayer.h"
-#include "playermanager.h"
 #include "playlistinterface.h"
+#include "playeradaptor.h"
 #include "slideraction.h"
 #include "statuslabel.h"
 #include "actioncollection.h"
@@ -42,76 +48,38 @@
 #include "coverinfo.h"
 #include "tag.h"
 
-#include "config.h"
-
 using namespace ActionCollection;
 
 enum PlayerManagerStatus { StatusStopped = -1, StatusPaused = 1, StatusPlaying = 2 };
 
-////////////////////////////////////////////////////////////////////////////////
-// helper functions
-////////////////////////////////////////////////////////////////////////////////
-
-enum SoundSystem { ArtsBackend = 0, GStreamerBackend = 1, AkodeBackend = 2 };
-
-static Player *createPlayer(int system = ArtsBackend)
-{
-
-    Player *p = 0;
-    switch(system) {
-#ifdef HAVE_AKODE
-    case AkodeBackend:
-        p = new aKodePlayer;
-        break;
-#endif
-#if HAVE_ARTS
-    case ArtsBackend:
-        p = new ArtsPlayer;
-        break;
-#endif
-#if HAVE_GSTREAMER
-    case GStreamerBackend:
-        p = new GStreamerPlayer;
-        break;
-#endif 
-    default:
-#if HAVE_ARTS
-        p = new ArtsPlayer;
-#elif HAVE_GSTREAMER
-        p = new GStreamerPlayer;
-#else
-        p = new aKodePlayer;
-#endif
-        break;
-    }
-    return p;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // protected members
 ////////////////////////////////////////////////////////////////////////////////
 
 PlayerManager::PlayerManager() :
-    Player(),
+    QObject(),
     m_sliderAction(0),
     m_playlistInterface(0),
     m_statusLabel(0),
-    m_player(0),
-    m_timer(0),
     m_noSeek(false),
     m_muted(false),
-    m_setup(false)
+    m_setup(false),
+    m_output(0),
+    m_media(0),
+    m_fader(0)
 {
 // This class is the first thing constructed during program startup, and
 // therefore has no access to the widgets needed by the setup() method.
 // Since the setup() method will be called indirectly by the player() method
 // later, just disable it here. -- mpyne
 //    setup();
+    new PlayerAdaptor( this );
+    QDBusConnection::sessionBus().registerObject("/Player", this);
+
 }
 
 PlayerManager::~PlayerManager()
 {
-    delete m_player;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -126,37 +94,37 @@ PlayerManager *PlayerManager::instance() // static
 
 bool PlayerManager::playing() const
 {
-    if(!player())
+    if(!m_media)
         return false;
 
-    return player()->playing();
+    return (m_media->state() == Phonon::PlayingState || m_media->state() == Phonon::BufferingState);
 }
 
 bool PlayerManager::paused() const
 {
-    if(!player())
+    if(!m_media)
         return false;
 
-    return player()->paused();
+    return m_media->state() == Phonon::PausedState;
 }
 
 float PlayerManager::volume() const
 {
-    if(!player())
-        return 0;
+    if(!m_output)
+        return 1.0;
 
-    return player()->volume();
+    return m_output->volume();
 }
 
 int PlayerManager::status() const
 {
-    if(!player())
+    if(!m_media)
         return StatusStopped;
 
-    if(player()->paused())
+    if(paused())
         return StatusPaused;
 
-    if(player()->playing())
+    if(playing())
         return StatusPlaying;
 
     return 0;
@@ -164,27 +132,32 @@ int PlayerManager::status() const
 
 int PlayerManager::totalTime() const
 {
-    if(!player())
+    if(!m_media)
         return 0;
 
-    return player()->totalTime();
+    return m_media->totalTime() / 1000;
 }
 
 int PlayerManager::currentTime() const
 {
-    if(!player())
+    if(!m_media)
         return 0;
 
-    return player()->currentTime();
+    return m_media->currentTime() / 1000;
 }
 
+/*
 int PlayerManager::position() const
 {
-    if(!player())
+    if(!m_media)
         return 0;
 
-    return player()->position();
+    long curr = m_media->currentTime();
+    if(curr > 0)
+        return static_cast<int>(static_cast<float>(curr * SliderAction::maxPosition) / m_media->totalTime() + 0.5f);
+    return -1;
 }
+*/
 
 QStringList PlayerManager::trackProperties()
 {
@@ -194,7 +167,7 @@ QStringList PlayerManager::trackProperties()
 QString PlayerManager::trackProperty(const QString &property) const
 {
     if(!playing() && !paused())
-        return QString::null;
+        return QString();
 
     return m_file.property(property);
 }
@@ -204,9 +177,9 @@ QPixmap PlayerManager::trackCover(const QString &size) const
     if(!playing() && !paused())
         return QPixmap();
 
-    if(size.lower() == "small")
+    if(size.toLower() == "small")
         return m_file.coverInfo()->pixmap(CoverInfo::Thumbnail);
-    if(size.lower() == "large")
+    if(size.toLower() == "large")
         return m_file.coverInfo()->pixmap(CoverInfo::FullSize);
 
     return QPixmap();
@@ -220,7 +193,7 @@ FileHandle PlayerManager::playingFile() const
 QString PlayerManager::playingString() const
 {
     if(!playing())
-        return QString::null;
+        return QString();
 
     QString str = m_file.tag()->artist() + " - " + m_file.tag()->title();
     if(m_file.tag()->artist().isEmpty())
@@ -239,82 +212,52 @@ void PlayerManager::setStatusLabel(StatusLabel *label)
     m_statusLabel = label;
 }
 
-KSelectAction *PlayerManager::playerSelectAction(QObject *parent) // static
-{
-    KSelectAction *action = 0;
-    action = new KSelectAction(i18n("&Output To"), 0, parent, "outputSelect");
-    QStringList l;
-
-#if HAVE_ARTS
-    l << i18n("aRts");
-#endif
-#if HAVE_GSTREAMER
-    l << i18n("GStreamer");
-#endif
-#ifdef HAVE_AKODE
-    l << i18n("aKode");
-#endif
-
-    if(l.isEmpty()) {
-        kdError(65432) << "Your JuK seems to have no output backend possibilities.\n";
-        l << i18n("aKode"); // Looks like akode is the default backend.
-    }
-
-    action->setItems(l);
-    return action;
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // public slots
 ////////////////////////////////////////////////////////////////////////////////
 
 void PlayerManager::play(const FileHandle &file)
 {
-    if(!player() || !m_playlistInterface)
+    if(!m_media)
+        setup();
+
+    if(!m_media || !m_playlistInterface)
         return;
 
+    // Ensure we're not trying to fade if we were previously stopped or end
+    // up not crossfading.
+    m_fader->setVolume(volume());
+
     if(file.isNull()) {
-        if(player()->paused())
-            player()->play();
-        else if(player()->playing()) {
-            if(m_sliderAction->trackPositionSlider())
-                m_sliderAction->trackPositionSlider()->setValue(0);
-            player()->seekPosition(0);
+        if(paused())
+            m_media->play();
+        else if(playing()) {
+            m_media->seek(0);
         }
         else {
             m_playlistInterface->playNext();
             m_file = m_playlistInterface->currentFile();
 
             if(!m_file.isNull())
-                player()->play(m_file);
+            {
+                m_media->setCurrentSource(KUrl::fromPath(m_file.absFilePath()));
+                m_media->play();
+            }
         }
     }
     else {
         m_file = file;
-        player()->play(file);
+
+        if(m_media->state() == Phonon::PlayingState)
+            crossfadeToFile(m_file);
+        else {
+            m_media->setCurrentSource(KUrl::fromPath(m_file.absFilePath()));
+            m_media->play();
+        }
     }
 
-    // Make sure that the player() actually starts before doing anything.
-
-    if(!player()->playing()) {
-        kdWarning(65432) << "Unable to play " << file.absFilePath() << endl;
-        stop();
-        return;
-    }
-
-    action("pause")->setEnabled(true);
-    action("stop")->setEnabled(true);
-    action("forward")->setEnabled(true);
-    if(action<KToggleAction>("albumRandomPlay")->isChecked())
-        action("forwardAlbum")->setEnabled(true);
-    action("back")->setEnabled(true);
-
-    if(m_sliderAction->trackPositionSlider())
-        m_sliderAction->trackPositionSlider()->setEnabled(true);
-
-    m_timer->start(m_pollInterval);
-
-    emit signalPlay();
+    // Our state changed handler will perform the follow up actions necessary
+    // once we actually start playing.
 }
 
 void PlayerManager::play(const QString &file)
@@ -333,92 +276,100 @@ void PlayerManager::play()
 
 void PlayerManager::pause()
 {
-    if(!player())
+    if(!m_media)
         return;
 
-    if(player()->paused()) {
+    if(paused()) {
         play();
         return;
     }
 
-    m_timer->stop();
     action("pause")->setEnabled(false);
 
-    player()->pause();
+    m_media->pause();
 
     emit signalPause();
 }
 
 void PlayerManager::stop()
 {
-    if(!player() || !m_playlistInterface)
+    if(!m_media || !m_playlistInterface)
         return;
 
-    m_timer->stop();
-
     action("pause")->setEnabled(false);
     action("stop")->setEnabled(false);
     action("back")->setEnabled(false);
     action("forward")->setEnabled(false);
     action("forwardAlbum")->setEnabled(false);
 
-    if(m_sliderAction->trackPositionSlider()) {
-        m_sliderAction->trackPositionSlider()->setValue(0);
-        m_sliderAction->trackPositionSlider()->setEnabled(false);
+    switch(m_media->state())
+    {
+    case Phonon::PlayingState:
+        // Fall through
+
+    case Phonon::BufferingState:
+    {
+        // Effect will be deleted once we actually stop playbackk.
+        kDebug() << "Fading out playback.\n";
+
+        m_fader->setFadeCurve(Phonon::VolumeFaderEffect::Fade12Decibel);
+        m_fader->fadeOut(2000);
+        QTimer::singleShot(2000, m_media, SLOT(stop()));
     }
 
-    player()->stop();
-    m_playlistInterface->stop();
+    break;
 
-    m_file = FileHandle::null();
-
-    emit signalStop();
+    default:
+        m_media->stop();
+    }
 }
 
 void PlayerManager::setVolume(float volume)
 {
-    if(!player())
-        return;
-
-    player()->setVolume(volume);
+    if(!m_output)
+        setup();
+    m_output->setVolume(volume);
 }
 
 void PlayerManager::seek(int seekTime)
 {
-    if(!player())
+    if(!m_media)
         return;
 
-    player()->seek(seekTime);
+    m_media->seek(seekTime);
 }
 
+/*
 void PlayerManager::seekPosition(int position)
 {
-    if(!player())
+    if(!m_media)
         return;
 
-    if(!player()->playing() || m_noSeek)
+    if(!playing() || m_noSeek)
         return;
 
     slotUpdateTime(position);
-    player()->seekPosition(position);
-
-    if(m_sliderAction->trackPositionSlider())
-        m_sliderAction->trackPositionSlider()->setValue(position);
+    m_media->seek(static_cast<qint64>(static_cast<float>(m_media->totalTime() * position) / SliderAction::maxPosition + 0.5f));
 }
+*/
 
 void PlayerManager::seekForward()
 {
-    seekPosition(kMin(SliderAction::maxPosition, position() + 10));
+    const qint64 total = m_media->totalTime();
+    const qint64 newtime = m_media->currentTime() + total / 100;
+    m_media->seek(qMin(total, newtime));
 }
 
 void PlayerManager::seekBack()
 {
-    seekPosition(kMax(SliderAction::minPosition, position() - 10));
+    const qint64 total = m_media->totalTime();
+    const qint64 newtime = m_media->currentTime() - total / 100;
+    m_media->seek(qMax(qint64(0), newtime));
 }
 
 void PlayerManager::playPause()
 {
-    playing() ? action("pause")->activate() : action("play")->activate();
+    playing() ? action("pause")->trigger() : action("play")->trigger();
 }
 
 void PlayerManager::forward()
@@ -445,161 +396,151 @@ void PlayerManager::back()
 
 void PlayerManager::volumeUp()
 {
-    if(!player() || !m_sliderAction || !m_sliderAction->volumeSlider())
+    if(!m_output)
         return;
 
-    int volume = m_sliderAction->volumeSlider()->volume() +
-        m_sliderAction->volumeSlider()->maxValue() / 25; // 4% up
-
-    slotSetVolume(volume);
-    m_sliderAction->volumeSlider()->setVolume(volume);
+    float volume = m_output->volume() + 0.04; // 4% up
+    m_output->setVolume(volume);
 }
 
 void PlayerManager::volumeDown()
 {
-    if(!player() || !m_sliderAction || !m_sliderAction->volumeSlider())
+    if(!m_output)
         return;
 
-    int volume = m_sliderAction->volumeSlider()->value() -
-        m_sliderAction->volumeSlider()->maxValue() / 25; // 4% down
-
-    slotSetVolume(volume);
-    m_sliderAction->volumeSlider()->setVolume(volume);
+    float volume = m_output->volume() - 0.04; // 4% up
+    m_output->setVolume(volume);
 }
 
 void PlayerManager::mute()
 {
-    if(!player() || !m_sliderAction || !m_sliderAction->volumeSlider())
+    if(!m_output)
         return;
 
-    slotSetVolume(m_muted ? m_sliderAction->volumeSlider()->value() : 0);
-    m_muted = !m_muted;
+    m_output->setMuted(!m_output->isMuted());
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // private slots
 ////////////////////////////////////////////////////////////////////////////////
 
-void PlayerManager::slotPollPlay()
+void PlayerManager::slotNeedNextUrl()
 {
-    if(!player() || !m_playlistInterface)
+    if(m_file.isNull())
+    {
+        return;
+    }
+    m_playlistInterface->playNext();
+    FileHandle nextFile = m_playlistInterface->currentFile();
+    if(!nextFile.isNull())
+    {
+        //kDebug() << m_file.absFilePath();
+        m_file = nextFile;
+        crossfadeToFile(m_file);
+    }
+}
+
+void PlayerManager::slotFinished()
+{
+    action("pause")->setEnabled(false);
+    action("stop")->setEnabled(false);
+    action("back")->setEnabled(false);
+    action("forward")->setEnabled(false);
+    action("forwardAlbum")->setEnabled(false);
+
+    m_playlistInterface->stop();
+
+    m_file = FileHandle::null();
+
+    emit signalStop();
+}
+
+void PlayerManager::slotLength(qint64 msec)
+{
+    m_statusLabel->setItemTotalTime(msec / 1000);
+}
+
+void PlayerManager::slotTick(qint64 msec)
+{
+    if(!m_media || !m_playlistInterface)
         return;
 
     m_noSeek = true;
 
-    if(!player()->playing()) {
-        m_timer->stop();
-
-        m_playlistInterface->playNext();
-        FileHandle nextFile = m_playlistInterface->currentFile();
-        if(!nextFile.isNull())
-            play(nextFile);
-        else
-            stop();
-    }
-    else if(!m_sliderAction->dragging()) {
-        if(m_sliderAction->trackPositionSlider())
-            m_sliderAction->trackPositionSlider()->setValue(player()->position());
-
-        if(m_statusLabel) {
-            m_statusLabel->setItemTotalTime(player()->totalTime());
-            m_statusLabel->setItemCurrentTime(player()->currentTime());
-        }
-    }
-
-    // This call is done because when the user adds the slider to the toolbar
-    // while playback is occuring the volume slider generally defaults to 0,
-    // and doesn't get updated to the correct volume.  It might be better to
-    // have the SliderAction class fill in the correct volume, but I'm trying
-    // to avoid having it depend on PlayerManager since it may not be
-    // constructed in time during startup. -mpyne
-
-    if(!m_sliderAction->volumeDragging() && m_sliderAction->volumeSlider())
-    {
-        int maxV = m_sliderAction->volumeSlider()->maxValue();
-        float v = sqrt(sqrt(volume())); // Cancel out exponential scaling
-
-        m_sliderAction->volumeSlider()->blockSignals(true);
-        m_sliderAction->volumeSlider()->setVolume((int)((v) * maxV));
-        m_sliderAction->volumeSlider()->blockSignals(false);
-    }
-
-    // Ok, this is weird stuff, but it works pretty well.  Ordinarily we don't
-    // need to check up on our playing time very often, but in the span of the
-    // last interval, we want to check a lot -- to figure out that we've hit the
-    // end of the song as soon as possible.
-
-    if(player()->playing() &&
-       player()->totalTime() > 0 &&
-       float(player()->totalTime() - player()->currentTime()) < m_pollInterval * 2)
-    {
-        m_timer->changeInterval(50);
+    if(m_statusLabel) {
+        m_statusLabel->setItemCurrentTime(msec / 1000);
     }
 
     m_noSeek = false;
 }
 
-void PlayerManager::slotSetOutput(const QString &system)
+void PlayerManager::slotStateChanged(Phonon::State newstate)
 {
-    stop();
-    setOutput(system);
-    setup();
-}
+    if(newstate == Phonon::ErrorState) {
+        switch(m_media->errorType()) {
+            case Phonon::NoError:
+                kDebug() << "received a state change to ErrorState but errorType is NoError!?";
+                break;
 
-void PlayerManager::setOutput(const QString &system)
-{
-    delete m_player;
-    if(system == i18n("aRts"))
-        m_player = createPlayer(ArtsBackend);
-    else if(system == i18n("GStreamer"))
-        m_player = createPlayer(GStreamerBackend);
-    else if(system == i18n("aKode"))
-        m_player = createPlayer(AkodeBackend);
-}
+            case Phonon::NormalError:
+                forward();
+                KMessageBox::information(0, m_media->errorString());
+                break;
 
-void PlayerManager::slotSetVolume(int volume)
-{
-    float scaledVolume;
-
-    if(m_sliderAction->volumeSlider())
-        scaledVolume = float(volume) / m_sliderAction->volumeSlider()->maxValue();
-    else {
-        scaledVolume = float(volume) / 100.0; // Hopefully this is accurate
-        scaledVolume = kMin(1.0f, scaledVolume);
+            case Phonon::FatalError:
+                // stop playback
+                stop();
+                KMessageBox::sorry(0, m_media->errorString());
+                break;
+        }
     }
+    else if(newstate == Phonon::StoppedState) {
+        m_playlistInterface->stop();
 
-    // Perform exponential scaling to counteract the fact that humans perceive
-    // volume changes logarithmically.
+        m_file = FileHandle::null();
 
-    scaledVolume *= scaledVolume;
-    scaledVolume *= scaledVolume;
-    setVolume(scaledVolume); // scaledVolume ^ 4
+        emit signalStop();
+    }
+    else if(newstate == Phonon::PlayingState) {
+        action("pause")->setEnabled(true);
+        action("stop")->setEnabled(true);
+        action("forward")->setEnabled(true);
+        if(action<KToggleAction>("albumRandomPlay")->isChecked())
+            action("forwardAlbum")->setEnabled(true);
+        action("back")->setEnabled(true);
+
+        emit signalPlay();
+    }
 }
 
+void PlayerManager::slotKillSender()
+{
+    // When called as a slot the sender() property has a QObject that called us.
+    // If it's a MediaObject we should delete it to clean up from crossfading.
+    // Note: This is not a good general practice (deleting our senders) and is dependant on what
+    // MediaObject does.  So we use deleteLater() to delete at some later time.
+    if(qobject_cast<Phonon::MediaObject *>(sender()))
+        sender()->deleteLater();
+}
+
+/*
 void PlayerManager::slotUpdateTime(int position)
 {
     if(!m_statusLabel)
         return;
 
     float positionFraction = float(position) / SliderAction::maxPosition;
-    float totalTime = float(player()->totalTime());
+    float totalTime = float(m_media->totalTime()) / 1000.0f;
     int seekTime = int(positionFraction * totalTime + 0.5); // "+0.5" for rounding
 
     m_statusLabel->setItemCurrentTime(seekTime);
 }
+*/
 
 ////////////////////////////////////////////////////////////////////////////////
 // private members
 ////////////////////////////////////////////////////////////////////////////////
 
-Player *PlayerManager::player() const
-{
-    if(!m_player)
-        instance()->setup();
-
-    return m_player;
-}
-
 void PlayerManager::setup()
 {
     // All of the actions required by this class should be listed here.
@@ -611,7 +552,7 @@ void PlayerManager::setup()
        !action("forward") ||
        !action("trackPositionAction"))
     {
-        kdWarning(65432) << k_funcinfo << "Could not find all of the required actions." << endl;
+        kWarning(65432) << "Could not find all of the required actions.";
         return;
     }
 
@@ -619,6 +560,19 @@ void PlayerManager::setup()
         return;
     m_setup = true;
 
+    m_output = new Phonon::AudioOutput(Phonon::MusicCategory, this);
+
+    m_media = new Phonon::MediaObject(this);
+    connect(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)), SLOT(slotStateChanged(Phonon::State)));
+    connect(m_media, SIGNAL(aboutToFinish()), SLOT(slotNeedNextUrl()));
+    m_audioPath = Phonon::createPath(m_media, m_output);
+    m_media->setTickInterval(200);
+
+    // Pre-cache a volume fader object
+    m_fader = new Phonon::VolumeFaderEffect(m_media);
+    m_audioPath.insertEffect(m_fader);
+    m_fader->setVolume(volume());
+
     // initialize action states
 
     action("pause")->setEnabled(false);
@@ -631,38 +585,76 @@ void PlayerManager::setup()
 
     m_sliderAction = action<SliderAction>("trackPositionAction");
 
+    /*
     connect(m_sliderAction, SIGNAL(signalPositionChanged(int)),
             this, SLOT(seekPosition(int)));
     connect(m_sliderAction->trackPositionSlider(), SIGNAL(valueChanged(int)),
             this, SLOT(slotUpdateTime(int)));
-    connect(m_sliderAction, SIGNAL(signalVolumeChanged(int)),
-            this, SLOT(slotSetVolume(int)));
+            */
 
-    // Call this method manually to avoid warnings.
-
-    KAction *outputAction = actions()->action("outputSelect");
-
-    if(outputAction) {
-        setOutput(static_cast<KSelectAction *>(outputAction)->currentText());
-        connect(outputAction, SIGNAL(activated(const QString &)), this, SLOT(slotSetOutput(const QString &)));
+    if(m_sliderAction->trackPositionSlider())
+    {
+        m_sliderAction->trackPositionSlider()->setMediaObject(m_media);
     }
-    else
-        m_player = createPlayer();
-
-    float volume;
-    
-    if(m_sliderAction->volumeSlider()) {
-        volume = 
-            float(m_sliderAction->volumeSlider()->volume()) /
-            float(m_sliderAction->volumeSlider()->maxValue());
+    if(m_sliderAction->volumeSlider())
+    {
+        m_sliderAction->volumeSlider()->setAudioOutput(m_output);
     }
-    else
-        volume = 1; // Assume user wants full volume
 
-    m_player->setVolume(volume);
+    connect(m_media, SIGNAL(totalTimeChanged(qint64)), SLOT(slotLength(qint64)));
+    connect(m_media, SIGNAL(tick(qint64)), SLOT(slotTick(qint64)));
+    connect(m_media, SIGNAL(finished()), SLOT(slotFinished()));
+}
 
-    m_timer = new QTimer(this, "play timer");
-    connect(m_timer, SIGNAL(timeout()), this, SLOT(slotPollPlay()));
+void PlayerManager::crossfadeToFile(const FileHandle &newFile)
+{
+    // Crossfade by introducting a second MediaObject, inserting appropriate effects in the new
+    // and old MediaObject, and letting the old MediaObject die a graceful death afterwards.
+
+    Phonon::VolumeFaderEffect *oldFader = m_fader;
+    Phonon::Path oldPath(m_audioPath);
+    Phonon::MediaObject *mo = m_media;
+    Phonon::AudioOutput *out = m_output;
+
+    mo->disconnect(this);
+    out->setParent(mo); // Allow mo's death to also kill out
+    connect(mo, SIGNAL(finished()), SLOT(slotKillSender()));
+
+    m_media = new Phonon::MediaObject(this);
+    m_output = new Phonon::AudioOutput(Phonon::MusicCategory, this);
+    m_fader = new Phonon::VolumeFaderEffect(m_media);
+
+    m_audioPath = Phonon::createPath(m_media, m_output);
+    m_output->setVolume(out->volume());
+    m_output->setMuted(out->isMuted());
+    m_media->setTickInterval(200);
+    m_audioPath.insertEffect(m_fader);
+
+    // Reconnect everything
+    connect(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)), SLOT(slotStateChanged(Phonon::State)));
+    connect(m_media, SIGNAL(aboutToFinish()), SLOT(slotNeedNextUrl()));
+    if(m_sliderAction->trackPositionSlider())
+        m_sliderAction->trackPositionSlider()->setMediaObject(m_media);
+    connect(m_media, SIGNAL(totalTimeChanged(qint64)), SLOT(slotLength(qint64)));
+    connect(m_media, SIGNAL(tick(qint64)), SLOT(slotTick(qint64)));
+    connect(m_media, SIGNAL(finished()), SLOT(slotFinished()));
+
+    if(m_sliderAction->trackPositionSlider())
+        m_sliderAction->trackPositionSlider()->setMediaObject(m_media);
+
+    if(m_sliderAction->volumeSlider())
+        m_sliderAction->volumeSlider()->setAudioOutput(m_output);
+
+    m_media->setCurrentSource(newFile.absFilePath());
+    m_media->play();
+
+    oldFader->fadeTo(0.0f, 2000);
+
+    m_fader->setVolume(0.0f);
+    m_fader->fadeTo(m_output->volume(), 2000);
+
+    // Give the media object some extra time for good measure but then kill it.
+    QTimer::singleShot(4000, mo, SLOT(deleteLater()));
 }
 
 QString PlayerManager::randomPlayMode() const
@@ -676,14 +668,14 @@ QString PlayerManager::randomPlayMode() const
 
 void PlayerManager::setRandomPlayMode(const QString &randomMode)
 {
-    if(randomMode.lower() == "random")
+    if(randomMode.toLower() == "random")
         action<KToggleAction>("randomPlay")->setChecked(true);
-    if(randomMode.lower() == "albumrandom")
+    if(randomMode.toLower() == "albumrandom")
         action<KToggleAction>("albumRandomPlay")->setChecked(true);
-    if(randomMode.lower() == "norandom")
+    if(randomMode.toLower() == "norandom")
         action<KToggleAction>("disableRandomPlay")->setChecked(true);
 }
 
 #include "playermanager.moc"
 
-// vim: set et ts=4 sw=4:
+// vim: set et sw=4 tw=0 sta:
