@@ -1,6 +1,6 @@
 /***************************************************************************
     begin                : Sat Jun 7 2003
-    copyright            : (C) 2003 - 2004 by Scott Wheeler, 
+    copyright            : (C) 2003 - 2004 by Scott Wheeler,
     email                : wheeler@kde.org
  ***************************************************************************/
 
@@ -13,17 +13,16 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "viewmode.h"
+
 #include <kiconloader.h>
 #include <kstandarddirs.h>
 #include <kdebug.h>
 
-#include <qpixmap.h>
-#include <qpainter.h>
-#include <qfile.h>
-#include <qdir.h>
-#include <qdatastream.h>
+#include <QPixmap>
+#include <QPainter>
+#include <QResizeEvent>
 
-#include "viewmode.h"
 #include "playlistbox.h"
 #include "searchplaylist.h"
 #include "treeviewitemplaylist.h"
@@ -48,7 +47,7 @@ ViewMode::~ViewMode()
 }
 
 void ViewMode::paintCell(PlaylistBox::Item *item,
-                         QPainter *painter, 
+                         QPainter *painter,
                          const QColorGroup &colorGroup,
                          int column, int width, int)
 {
@@ -71,30 +70,32 @@ void ViewMode::paintCell(PlaylistBox::Item *item,
         QPen newPen = oldPen;
 
         newPen.setWidth(5);
-        newPen.setJoinStyle(RoundJoin);
-        newPen.setColor(QColorGroup::Highlight);
+        newPen.setJoinStyle(Qt::RoundJoin);
+        newPen.setColor(QPalette::Highlight);
 
         painter->setPen(newPen);
         painter->drawRect(border, border, width - border * 2, item->height() - border * 2 + 1);
         painter->setPen(oldPen);
 
         painter->fillRect(border, border, width - border * 2, item->height() - border * 2 + 1,
-                          colorGroup.brush(QColorGroup::Highlight));
-        painter->setPen(colorGroup.highlightedText());
+                          colorGroup.brush(QPalette::Highlight));
+        painter->setPen(colorGroup.color( QPalette::HighlightedText));
     }
     else
         painter->eraseRect(0, 0, width, item->height());
 
-    if (!pm->isNull()) {
+    if(!pm->isNull()) {
         int x = (width - pm->width()) / 2;
-        x = QMAX(x, item->listView()->itemMargin());
+        x = qMax(x, item->listView()->itemMargin());
         painter->drawPixmap(x, y, *pm);
     }
+
     y += pm->height() + fm.height() - fm.descent();
-    for(QStringList::Iterator it = m_lines[item].begin(); it != m_lines[item].end(); ++it) {
-        int x = (width - fm.width(*it)) / 2;
-        x = QMAX(x, item->listView()->itemMargin());
-        painter->drawText(x, y, *it);
+
+    foreach(const QString &line, m_lines[item]) {
+        int x = (width - fm.width(line)) / 2;
+        x = qMax(x, item->listView()->itemMargin());
+        painter->drawText(x, y, line);
         y += fm.height() - fm.descent();
     }
 
@@ -116,6 +117,11 @@ bool ViewMode::eventFilter(QObject *watched, QEvent *e)
     return QObject::eventFilter(watched, e);
 }
 
+QString ViewMode::name() const
+{
+    return i18nc("the normal viewing mode", "Default");
+}
+
 void ViewMode::setShown(bool shown)
 {
     m_visible = shown;
@@ -127,7 +133,7 @@ void ViewMode::setShown(bool shown)
 
 void ViewMode::updateIcons(int size)
 {
-    for(QListViewItemIterator it(m_playlistBox); it.current(); ++it) {
+    for(Q3ListViewItemIterator it(m_playlistBox); it.current(); ++it) {
         PlaylistBox::Item *i = static_cast<PlaylistBox::Item *>(*it);
         i->setPixmap(0, SmallIcon(i->iconName(), size));
     }
@@ -149,7 +155,7 @@ void ViewMode::updateHeights()
     const int baseHeight = 2 * m_playlistBox->itemMargin() + 32 + border * 2;
     const QFontMetrics fm = m_playlistBox->fontMetrics();
 
-    for(QListViewItemIterator it(m_playlistBox); it.current(); ++it) {
+    for(Q3ListViewItemIterator it(m_playlistBox); it.current(); ++it) {
         PlaylistBox::Item *i = static_cast<PlaylistBox::Item *>(it.current());
         m_lines[i] = lines(i, fm, width);
         const int height = baseHeight + (fm.height() - fm.descent()) * m_lines[i].count();
@@ -168,7 +174,7 @@ void ViewMode::paintDropIndicator(QPainter *painter, int width, int height) // s
     QPen newPen = oldPen;
 
     newPen.setWidth(lineWidth);
-    newPen.setStyle(DotLine);
+    newPen.setStyle(Qt::DotLine);
 
     painter->setPen(newPen);
     painter->drawRect(border, border, width - border * 2, height - border * 2);
@@ -180,7 +186,7 @@ QStringList ViewMode::lines(const PlaylistBox::Item *item,
                             int width)
 {
     // Here 32 is a bit arbitrary, but that's the width of the icons in this
-    // mode and seems to a reasonable lower bound.
+    // mode and seems to a reasonable toLower bound.
 
     if(width < 32)
         return QStringList();
@@ -191,18 +197,18 @@ QStringList ViewMode::lines(const PlaylistBox::Item *item,
 
     while(!line.isEmpty()) {
         int textLength = line.length();
-        while(textLength > 0 && 
-              fm.width(line.mid(0, textLength).stripWhiteSpace()) +
+        while(textLength > 0 &&
+              fm.width(line.mid(0, textLength).trimmed()) +
               item->listView()->itemMargin() * 2 > width)
         {
-            int i = line.findRev(QRegExp( "\\W"), textLength - 1);
+            int i = line.lastIndexOf(QRegExp( "\\W"), textLength - 1);
             if(i > 0)
                 textLength = i;
             else
                 textLength--;
         }
-        
-        l.append(line.mid(0, textLength).stripWhiteSpace());
+
+        l.append(line.mid(0, textLength).trimmed());
         line = line.mid(textLength);
     }
     return l;
@@ -223,15 +229,20 @@ CompactViewMode::~CompactViewMode()
 }
 
 void CompactViewMode::paintCell(PlaylistBox::Item *item,
-                                QPainter *painter, 
+                                QPainter *painter,
                                 const QColorGroup &colorGroup,
                                 int column, int width, int align)
 {
-    item->KListViewItem::paintCell(painter, colorGroup, column, width, align);
+    item->K3ListViewItem::paintCell(painter, colorGroup, column, width, align);
     if(item == item->listView()->dropItem())
         paintDropIndicator(painter, width, item->height());
 }
 
+QString CompactViewMode::name() const
+{
+    return i18nc("compact viewing mode", "Compact");
+}
+
 void CompactViewMode::setShown(bool shown)
 {
     setVisible(shown);
@@ -244,7 +255,7 @@ void CompactViewMode::setShown(bool shown)
 
 void CompactViewMode::updateHeights()
 {
-    for(QListViewItemIterator it(playlistBox()); it.current(); ++it)
+    for(Q3ListViewItemIterator it(playlistBox()); it.current(); ++it)
         it.current()->setup();
 }
 
@@ -253,7 +264,7 @@ void CompactViewMode::updateHeights()
 ////////////////////////////////////////////////////////////////////////////////
 
 TreeViewMode::TreeViewMode(PlaylistBox *b) : CompactViewMode(b),
-    m_treeViewItems(5003, false), m_dynamicListsFrozen(false), m_setup(false)
+    m_dynamicListsFrozen(false), m_setup(false)
 {
 
 }
@@ -263,6 +274,11 @@ TreeViewMode::~TreeViewMode()
 
 }
 
+QString TreeViewMode::name() const
+{
+    return i18n("Tree");
+}
+
 void TreeViewMode::setShown(bool show)
 {
     CompactViewMode::setShown(show);
@@ -278,8 +294,8 @@ void TreeViewMode::setShown(bool show)
         if(collectionItem && m_searchCategories.isEmpty())
             setupDynamicPlaylists();
         else {
-            for(QDictIterator<PlaylistBox::Item> it(m_searchCategories); it.current(); ++it)
-                it.current()->setVisible(true);
+            foreach(PlaylistBox::Item *item, m_searchCategories)
+                item->setVisible(true);
         }
 
         if(!m_setup) {
@@ -291,8 +307,8 @@ void TreeViewMode::setShown(bool show)
         }
     }
     else {
-        for(QDictIterator<PlaylistBox::Item> it(m_searchCategories); it.current(); ++it)
-            it.current()->setVisible(false);
+        foreach(PlaylistBox::Item *item, m_searchCategories)
+            item->setVisible(false);
     }
 }
 
@@ -309,14 +325,14 @@ void TreeViewMode::removeItem(const QString &item, unsigned column)
     else if(column == PlaylistItem::AlbumColumn)
         itemKey = "albums" + item;
     else {
-        kdWarning() << k_funcinfo << "Unhandled column type " << column << endl;
+        kWarning() << "Unhandled column type " << column;
         return;
     }
 
-    if(!m_treeViewItems.find(itemKey))
+    if(!m_treeViewItems.contains(itemKey))
         return;
 
-    TreeViewItemPlaylist *itemPlaylist = m_treeViewItems[itemKey];
+    TreeViewItemPlaylist *itemPlaylist = m_treeViewItems.value(itemKey, 0);
 
     if(m_dynamicListsFrozen) {
         m_pendingItemsToRemove << itemKey;
@@ -341,11 +357,11 @@ void TreeViewMode::addItems(const QStringList &items, unsigned column)
     else if(column == PlaylistItem::AlbumColumn)
         searchCategory = "albums";
     else {
-        kdWarning() << k_funcinfo << "Unhandled column type " << column << endl;
+        kWarning() << "Unhandled column type " << column;
         return;
     }
 
-    QValueList<int> columns;
+    ColumnList columns;
     columns.append(column);
 
     PlaylistSearch::Component::MatchMode mode = PlaylistSearch::Component::ContainsWord;
@@ -356,23 +372,22 @@ void TreeViewMode::addItems(const QStringList &items, unsigned column)
     PlaylistList playlists;
     playlists.append(CollectionList::instance());
 
-    QString itemKey, item;
-    PlaylistBox::Item *itemParent = m_searchCategories[searchCategory];
+    QString itemKey;
+    PlaylistBox::Item *itemParent = m_searchCategories.value(searchCategory, 0);
 
-    for(QStringList::ConstIterator it = items.begin(); it != items.end(); ++it) {
-        item = *it;
+    foreach(const QString &item, items) {
         itemKey = searchCategory + item;
 
-        if(m_treeViewItems.find(itemKey))
+        if(m_treeViewItems.contains(itemKey))
             continue;
-            
+
         components.clear();
         components.append(PlaylistSearch::Component(item, false, columns, mode));
 
         PlaylistSearch s(playlists, components, PlaylistSearch::MatchAny, false);
 
         TreeViewItemPlaylist *p = new TreeViewItemPlaylist(playlistBox(), s, item);
-        playlistBox()->setupPlaylist(p, "midi", itemParent);
+        playlistBox()->setupPlaylist(p, "audio-midi", itemParent);
         m_treeViewItems.insert(itemKey, p);
     }
 }
@@ -384,15 +399,9 @@ void TreeViewMode::setDynamicListsFrozen(bool frozen)
     if(frozen)
         return;
 
-    QStringList categories;
-    categories << "artists" << "albums" << "genres";
-
-    for(QStringList::ConstIterator it = m_pendingItemsToRemove.begin();
-        it != m_pendingItemsToRemove.end();
-        ++it)
-    {
-        m_treeViewItems[*it]->deleteLater();
-        m_treeViewItems.remove(*it);
+    foreach(const QString &pendingItem, m_pendingItemsToRemove) {
+        m_treeViewItems[pendingItem]->deleteLater();
+        m_treeViewItems.remove(pendingItem);
     }
 
     m_pendingItemsToRemove.clear();
@@ -402,7 +411,7 @@ void TreeViewMode::setupDynamicPlaylists()
 {
     PlaylistBox::Item *i;
     PlaylistBox::Item *collectionItem = PlaylistBox::Item::collectionItem();
-    
+
     i = new PlaylistBox::Item(collectionItem, "cdimage", i18n("Artists"));
     m_searchCategories.insert("artists", i);
 
@@ -413,13 +422,6 @@ void TreeViewMode::setupDynamicPlaylists()
     m_searchCategories.insert("genres", i);
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// CoverManagerMode
-////////////////////////////////////////////////////////////////////////////////
-
-CoverManagerMode::CoverManagerMode(PlaylistBox *b) : ViewMode(b)
-{
-
-}
-
 #include "viewmode.moc"
+
+// vim: set et sw=4 tw=0 sta:
