@@ -1,7 +1,7 @@
 /*
  *              kPPP: A pppd Front End for the KDE project
  *
- * $Id: docking.h 295027 2004-03-10 19:24:30Z aseigo $
+ * $Id: docking.h 681320 2007-06-28 15:00:24Z kernalex $
  *
  *              Copyright (C) 1997 Bernd Johannes Wuebben
  *                      wuebben@math.cornell.edu
@@ -22,33 +22,27 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this program; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 
 #ifndef _DOCKING_H_
 #define _DOCKING_H_
 
-#include <stdio.h>
-#include <qapplication.h>
-#include <qpixmap.h>
-#include <qtimer.h>
-#include <qpopupmenu.h>
-#include <qpoint.h>
-#include <kdockwindow.h>
+#include <ksystemtrayicon.h>
+
+class QWheelEvent;
+class QAction;
 
 class KSCD;
-
-class KAction;
-class KToggleAction;
 class KPassivePopup;
 
-class DockWidget : public KSystemTray
+class DockWidget : public KSystemTrayIcon
 {
     Q_OBJECT
 
 public:
-    DockWidget( KSCD* parent, const char *name=0);
+    explicit DockWidget(KSCD* parent, const char *name = 0);
     ~DockWidget();
 
 public slots:
@@ -56,15 +50,12 @@ public slots:
     void createPopup(const QString& songName, bool addButtons = true);
 
 private:
-    virtual void wheelEvent( QWheelEvent *e);
+    virtual void wheelEvent(QWheelEvent *e);
 
     KPassivePopup* m_popup;
 
-    KAction* m_forwardAction;
-    KAction* m_backAction;
-
-    QPixmap m_backPix;
-    QPixmap m_forwardPix;
+    QAction* m_forwardAction;
+    QAction* m_backAction;
 
     QString tip;
 };
