@@ -15,13 +15,15 @@
 
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
-  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-  Boston, MA 02111-1307, USA.
+  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+  Boston, MA 02110-1301, USA.
 */
 
+#include "cddbplookup.h"
+
 #include <kdebug.h>
 
-#include "cddbplookup.h"
+#include <qbytearray.h>
 
 namespace KCDDB
 {
@@ -68,11 +70,11 @@ namespace KCDDB
   CDDBPLookup::sendRead( const CDDBMatch & match )
   {
     category_  = match.first;
-    QString discid    = match.second;
+    discid_    = match.second;
 
     QString readRequest = QString( "cddb read %1 %2" )
         .arg( category_ )
-        .arg( discid );
+        .arg( discid_ );
 
     writeLine( readRequest );
   }
@@ -86,27 +88,67 @@ namespace KCDDB
     void
   CDDBPLookup::close()
   {
-    kdDebug(60010) << "Disconnect from server..." << endl;
+    kDebug(60010) << "Disconnect from server...";
     if ( isConnected() )
     {
       socket_->close();
     }
   }
 
+    bool
+  CDDBPLookup::parseGreeting( const QString & line )
+  {
+    uint serverStatus = statusCode( line );
+
+    if ( 200 == serverStatus )
+    {
+      kDebug(60010) << "Server response: read-only";
+      readOnly_ = true;
+    }
+    else if ( 201 == serverStatus )
+    {
+      kDebug(60010) << "Server response: read-write";
+    }
+    else
+    {
+      kDebug(60010) << "Server response: bugger off";
+      return false;
+    }
+
+    return true;
+  }
+
+    bool
+  CDDBPLookup::parseHandshake( const QString & line )
+  {
+    uint serverStatus = statusCode( line );
+
+    if ( ( 200 != serverStatus ) && ( 402 != serverStatus ) )
+    {
+      kDebug(60010) << "Handshake was too tight. Letting go.";
+      return false;
+    }
+
+    kDebug(60010) << "Handshake was warm and firm";
+
+    return true;
+  }
+
+
     Q_LONG
   CDDBPLookup::writeLine( const QString & line )
   {
     if ( !isConnected() )
     {
-      kdDebug(60010) << "socket status: " << socket_->state() << endl;
+      kDebug(60010) << "socket status: " << socket_->state();
       return -1;
     }
 
-    kdDebug(60010) << "WRITE: [" << line << "]" << endl;
-    QCString buf = line.utf8();
-    buf.append( "\n" );
+    kDebug(60010) << "WRITE: [" << line << "]";
+    QByteArray buf(line.toUtf8());
+    buf.append( '\n' );
 
-    return socket_->writeBlock( buf.data(), buf.length() );
+    return socket_->write( buf );
   }
 }
 
